"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TSExpectAgent = exports.TSUnaryTest = exports.TSTestGroup = exports.TSTest = exports.TSTester = void 0;
const util_1 = require("util");
const commons_1 = require("./commons");
const compare_1 = require("./compare");
const types_1 = require("./types");
const utils_1 = require("./utils");
class TSTester {
    constructor(s) {
        this.groups = [];
        this.desc = s;
    }
    addGroup(grp) { this.addGroups([grp]); }
    addGroups(grps) {
        if (grps instanceof TSTestGroup) {
            grps = [grps];
        }
        for (let g of grps) {
            this.groups.push(g);
        }
    }
    log(format, ...args) {
        (0, utils_1.$logterm)(format, args);
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            // first phase, we construct all the unary test in an asynchronus function call
            this.log(`&Z&eTSTester will now start &Y&k ${this.desc} &0&e :`);
            let expectations = 0;
            let expectationsFailed = 0;
            for (let g of this.groups) {
                yield g.prepare();
            }
            let passed = 0, failed = 0;
            // second phase, we run the tests
            for (let g of this.groups) {
                const [groupExpected, groupFailed] = yield g.run();
                expectations += groupExpected;
                expectationsFailed += groupFailed;
                if (groupFailed === 0) {
                    passed++;
                }
                else {
                    failed++;
                }
            }
            this.log(`&y${expectations.toString().padStart(4)}&0&e test${expectations === 1 ? 'was' : 's were'} executed.&0`);
            if (passed > 0) {
                this.log(`&g${passed.toString().padStart(4)}&0&j group${passed === 1 ? ' ' : 's'} of tests did &G&w  PASS  &0`);
            }
            if (failed > 0) {
                this.log(`&r${failed.toString().padStart(4)}&0&o group${failed === 1 ? ' ' : 's'} of tests did &R&w  FAIL  &0`);
            }
            if (passed > 0 && !failed) {
                this.log('&G&w  ALL TESTS PASSED  &0');
            }
            else if (expectationsFailed > 0) {
                this.log(`&R&w${expectationsFailed.toString().padStart(4)} TEST${expectationsFailed > 1 ? 'S' : ''} FAILED  &0`);
            }
        });
    }
}
exports.TSTester = TSTester;
class TSTest {
    constructor(s, f) {
        this.desc = s;
        this.fn = f;
    }
    static group(s, f) {
        return new TSTestGroup(s, f);
    }
}
exports.TSTest = TSTest;
class TSTestGroup extends TSTest {
    constructor(s, f) {
        super(s, f);
        this._unaries = [];
    }
    unary(s, f) {
        this._unaries.push(new TSUnaryTest(this, s, f));
    }
    prepare() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fn(this);
        });
    }
    log(format, ...args) {
        (0, utils_1.$logterm)('  ' + format, args);
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            let expectations = 0;
            let expectationFailed = 0;
            this.log(`&u- Running group tests &U&k ${this.desc} &0`);
            for (let u of this._unaries) {
                const [unaryExpected, unaryFailed] = yield u.run();
                expectationFailed += unaryFailed;
                expectations += unaryExpected;
            }
            (0, utils_1.$logterm)(`${expectationFailed == 0 ? '&j' : '\n&o'}    ${this.desc} tests ${expectationFailed == 0 ? '&G&w  PASSED  ' : '&R&w  FAILED  '}&0\n`);
            return [expectations, expectationFailed];
        });
    }
}
exports.TSTestGroup = TSTestGroup;
class TSUnaryTest extends TSTest {
    constructor(g, s, f) {
        super(s, f);
        this._failed = 0;
        this._passed = 0;
        this._expected = 0;
        this._registrations = {};
        this._group = g;
    }
    log(format, ...args) {
        (0, utils_1.$logterm)('    ' + format, '', args);
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            this._passed = 0;
            this._failed = 0;
            (0, utils_1.$writeterm)(`&x     ➤ &ltesting &B&w ${this.desc} &0`);
            yield this.fn(this);
            if (this._failed > 0) {
                (0, utils_1.$writeterm)('\n     ');
                this.printRegistrations();
            }
            if (this._passed > 0) {
                (0, utils_1.$writeterm)(`&J&k ${this._passed} OK &0`);
            }
            if (this._failed > 0) {
                (0, utils_1.$writeterm)(`&R&w ${this._failed} KO &0`);
            }
            (0, utils_1.$logterm)('');
            return [this._expected, this._failed];
        });
    }
    register(name, value) {
        if ((0, commons_1.$defined)(value) && (0, commons_1.$length)(name)) {
            this._registrations[name] = value;
        }
    }
    printRegistrations() {
        const keys = (0, commons_1.$keys)(this._registrations);
        const n = keys.length;
        if (n === 1) {
            (0, utils_1.$logterm)(`\n&Y&b  REGISTERED ITEM &r&!${keys[0]}  &0&c\n${(0, utils_1.$inspect)(this._registrations[keys[0]])}&0\n`);
        }
        else if (n > 1) {
            (0, utils_1.$logterm)('\n&Y&b  REGISTERED ITEMS  &0');
            keys.forEach(k => { (0, utils_1.$logterm)(`&x  + &o&_&/&!${k}&0&x: &c${(0, utils_1.$inspect)(this._registrations[k])}&0`); });
            (0, utils_1.$logterm)('');
        }
    }
    expect(v, msg) {
        this._expected++;
        if (!(0, commons_1.$length)(msg)) {
            msg = this._expected.toString().padStart(4);
        }
        return new TSExpectAgent(this, v, msg);
    }
    // this could be considered as useless but eventually, I find it very usefull and literrate 
    expect0(v) { return this.expect(v, 'Line 0'); }
    expect1(v) { return this.expect(v, 'Line 1'); }
    expect2(v) { return this.expect(v, 'Line 2'); }
    expect3(v) { return this.expect(v, 'Line 3'); }
    expect4(v) { return this.expect(v, 'Line 4'); }
    expect5(v) { return this.expect(v, 'Line 5'); }
    expect6(v) { return this.expect(v, 'Line 6'); }
    expect7(v) { return this.expect(v, 'Line 7'); }
    expect8(v) { return this.expect(v, 'Line 8'); }
    expect9(v) { return this.expect(v, 'Line 9'); }
    expectA(v) { return this.expect(v, 'Line A'); }
    expectB(v) { return this.expect(v, 'Line B'); }
    expectC(v) { return this.expect(v, 'Line C'); }
    expectD(v) { return this.expect(v, 'Line D'); }
    expectE(v) { return this.expect(v, 'Line E'); }
    expectF(v) { return this.expect(v, 'Line F'); }
    expectG(v) { return this.expect(v, 'Line G'); }
    expectH(v) { return this.expect(v, 'Line H'); }
    expectI(v) { return this.expect(v, 'Line I'); }
    expectJ(v) { return this.expect(v, 'Line J'); }
    expectK(v) { return this.expect(v, 'Line K'); }
    expectL(v) { return this.expect(v, 'Line L'); }
    expectM(v) { return this.expect(v, 'Line M'); }
    expectN(v) { return this.expect(v, 'Line N'); }
    expectO(v) { return this.expect(v, 'Line O'); }
    expectP(v) { return this.expect(v, 'Line P'); }
    expectQ(v) { return this.expect(v, 'Line Q'); }
    expectR(v) { return this.expect(v, 'Line R'); }
    expectS(v) { return this.expect(v, 'Line S'); }
    expectT(v) { return this.expect(v, 'Line T'); }
    expectU(v) { return this.expect(v, 'Line U'); }
    expectV(v) { return this.expect(v, 'Line V'); }
    expectW(v) { return this.expect(v, 'Line W'); }
    expectX(v) { return this.expect(v, 'Line X'); }
    expectY(v) { return this.expect(v, 'Line Y'); }
    expectZ(v) { return this.expect(v, 'Line Z'); }
    fail() { this._failed++; }
    pass() { this._passed++; }
}
exports.TSUnaryTest = TSUnaryTest;
class TSExpectAgent {
    constructor(step, value, msg = '') {
        this._step = step;
        this._value = value;
        this._message = msg;
    }
    toBe(aValue) { var _a; if (!(0, compare_1.$equal)(aValue, this._value)) {
        this._elogfail(aValue);
    }
    else {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
    } }
    notToBe(aValue) { var _a; if ((0, compare_1.$equal)(aValue, this._value)) {
        this._nelogfail(aValue);
    }
    else {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
    } }
    toBeTruthy() { var _a; if (!this._value) {
        this._elogfail(true);
    }
    else {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
    } }
    toBeFalsy() { var _a; if (this._value) {
        this._elogfail(false);
    }
    else {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
    } }
    toBeUndefined() { var _a; if ((0, commons_1.$defined)(this._value)) {
        this._elogfail(undefined);
    }
    else {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
    } }
    toBeDefined() { var _a; if (!(0, commons_1.$defined)(this._value)) {
        this._nelogfail(undefined);
        ;
    }
    else {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
    } }
    toBeNull() { var _a; if (this._value !== null) {
        this._elogfail(null);
    }
    else {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
    } }
    toBeNotNull() { var _a; if (this._value === null) {
        this._nelogfail(null);
    }
    else {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
    } }
    toBeOK() { var _a; if (!(0, commons_1.$ok)(this._value)) {
        this._elogfail('<a non null defined value>');
    }
    else {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
    } }
    toBeNotOK() { var _a; if ((0, commons_1.$ok)(this._value)) {
        this._elogfail('<a null or undefined value>');
    }
    else {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
    } }
    toBeNaN() { var _a; if (!isNaN(this._value)) {
        this._elogfail(NaN);
    }
    else {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
    } }
    toBeANumber() { var _a; if (!(0, commons_1.$isnumber)(this._value)) {
        this._elogfail('<a valid number>');
    }
    else {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
    } }
    toBeAnInt() { var _a; if (!(0, commons_1.$isint)(this._value)) {
        this._elogfail('<an integer>');
    }
    else {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
    } }
    toBeAnUnsigned() { var _a; if (!(0, commons_1.$isunsigned)(this._value)) {
        this._elogfail('<an unsigned>');
    }
    else {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
    } }
    toBeAString() { var _a; if (!(0, commons_1.$isstring)(this._value)) {
        this._elogfail('<a string>');
    }
    else {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
    } }
    toBeBool() { var _a; if (!(0, commons_1.$isbool)(this._value)) {
        this._elogfail('<a boolean>');
    }
    else {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
    } }
    toBeAnEmail() { var _a; if (!(0, commons_1.$isemail)(this._value)) {
        this._elogfail('<an email>');
    }
    else {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
    } }
    toBeAnUrl() { var _a; if (!(0, commons_1.$isurl)(this._value)) {
        this._elogfail('<an url>');
    }
    else {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
    } }
    toBeAnUUID() { var _a; if (!(0, commons_1.$isuuid)(this._value)) {
        this._elogfail('<an UUID>');
    }
    else {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
    } }
    toBeAnObject() { var _a; if (!(0, commons_1.$isobject)(this._value)) {
        this._elogfail('<an object>');
    }
    else {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
    } }
    toBeArray() { var _a; if (!(0, commons_1.$isarray)(this._value)) {
        this._elogfail('<an array>');
    }
    else {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
    } }
    toBeADate() { var _a; if (!(0, commons_1.$isdate)(this._value)) {
        this._elogfail('<a date>');
    }
    else {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
    } }
    toBeAFunction() { var _a; if (!(0, commons_1.$isfunction)(this._value)) {
        this._elogfail('<a function>');
    }
    else {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
    } }
    gt(aValue) { var _a; if ((0, compare_1.$compare)(aValue, this._value) !== types_1.Ascending) {
        this._compfail(aValue, '>');
    }
    else {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
    } }
    gte(aValue) { var _a; if ((0, compare_1.$compare)(aValue, this._value) === types_1.Descending) {
        this._compfail(aValue, '≥');
    }
    else {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
    } }
    lt(aValue) { var _a; if ((0, compare_1.$compare)(aValue, this._value) !== types_1.Descending) {
        this._compfail(aValue, '<');
    }
    else {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
    } }
    lte(aValue) { var _a; if ((0, compare_1.$compare)(aValue, this._value) === types_1.Ascending) {
        this._compfail(aValue, '≤');
    }
    else {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
    } }
    _compfail(aValue, op) {
        var _a;
        const start = this._writeMessage();
        (0, utils_1.$logterm)(`&adid expect value:&O&w${(0, util_1.inspect)(this._value, false, 5)}&0`);
        (0, utils_1.$logterm)(`${start}&eto be ${op} to value:&E&b${(0, utils_1.$inspect)(aValue)}&0`);
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.fail();
    }
    _elogfail(aValue) {
        var _a;
        const start = this._writeMessage();
        (0, utils_1.$logterm)(`&edid expect value:&E&b${(0, utils_1.$inspect)(aValue)}&0`);
        (0, utils_1.$logterm)(`${start}&adid get as value:&O&w${(0, utils_1.$inspect)(this._value)}&0`);
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.fail();
    }
    _nelogfail(aValue) {
        var _a;
        this._writeMessage();
        (0, utils_1.$logterm)(`&adid not expect  :&O&w${(0, utils_1.$inspect)(aValue)}&0`);
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.fail();
    }
    _writeMessage() {
        const l = this._message.length;
        (0, utils_1.$writeterm)(`\n&y${this._message}&0 `);
        return l > 0 ? "".padStart(l + 1, " ") : "";
    }
}
exports.TSExpectAgent = TSExpectAgent;
//# sourceMappingURL=tstester.js.map